@@ -5,7 +5,7 @@ Arithmetic is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2020, Free Software Foundation, Inc.
+Copyright (C) 2015-2021, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -63,8 +63,10 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 /***************************************************************/
 /*************          Internal functions         *************/
 /***************************************************************/
-#define SET_NUM_OP(CTYPE) {                                \
-    CTYPE a=*(CTYPE *)(numpop->array); if(a>0) return a;    }
+#define SET_NUM_OP(CTYPE) {            \
+    CTYPE a=*(CTYPE *)(numpop->array); \
+    gal_data_free(numpop);             \
+    if(a>0) return a;    }
 
 static size_t
 pop_number_of_operands(struct arithmeticparams *p, int op, char *token_string,
@@ -512,7 +514,8 @@ wrapper_for_filter(struct arithmeticparams *p, char *token, int operator)
 
       /* Spin off threads for each pixel. */
       gal_threads_spin_off(arithmetic_filter, &afp, afp.input->size,
-                           p->cp.numthreads);
+                           p->cp.numthreads, p->cp.minmapsize,
+                           p->cp.quietmmap);
     }
 
 
@@ -839,8 +842,8 @@ arithmetic_interpolate_region(struct arithmeticparams *p,
 static void
 arithmetic_interpolate(struct arithmeticparams *p, int operator, char *token)
 {
-  int num_int, interpop;
   gal_data_t *interpolated;
+  int num_int, interpop=GAL_ARITHMETIC_OP_INVALID;
 
   /* First pop the number of nearby neighbors.*/
   gal_data_t *num = operands_pop(p, token);
@@ -930,7 +933,7 @@ arithmetic_collapse(struct arithmeticparams *p, char *token, int operator)
           "is %ld", dim);
   if(dim > input->ndim)
     error(EXIT_FAILURE, 0, "input dataset to '%s' has %zu dimension(s), "
-          "but you have asked to collapse along dimension %zu", token,
+          "but you have asked to collapse along dimension %ld", token,
           input->ndim, dim);
 
 
@@ -1343,6 +1346,28 @@ arithmetic_operator_run(struct arithmeticparams *p, int operator,
 
 
 
+/* Used by the 'set-' operator. */
+static int
+arithmetic_set_name_used_later(void *in, char *name)
+{
+  struct gal_arithmetic_set_params *p=(struct gal_arithmetic_set_params *)in;
+  gal_list_str_t *token, *tokens = (gal_list_str_t *)(p->tokens);
+
+  size_t counter=0;
+
+  /* If the name exists after the current token, then return 1. */
+  for(token=tokens;token!=NULL;token=token->next)
+    if( counter++ > p->tokencounter && !strcmp(token->v, name) )
+      return 1;
+
+  /* If we get to this point, it means that the name doesn't exist. */
+  return 0;
+}
+
+
+
+
+
 /* This function implements the reverse polish algorithm as explained
    in the Wikipedia page.
 
@@ -1357,11 +1382,14 @@ reversepolish(struct arithmeticparams *p)
   char *hdu, *filename, *printnum;
   int operator=GAL_ARITHMETIC_OP_INVALID;
 
-
   /* Prepare the processing: */
-  p->operands=NULL;
   p->popcounter=0;
-
+  p->operands=NULL;
+  p->setprm.params=p;
+  p->setprm.tokencounter=0;
+  p->setprm.tokens=p->tokens;
+  p->setprm.pop=operands_pop_wrapper_set;
+  p->setprm.used_later=arithmetic_set_name_used_later;
 
   /* Go over each input token and do the work. */
   for(token=p->tokens;token!=NULL;token=token->next)
@@ -1378,11 +1406,11 @@ reversepolish(struct arithmeticparams *p)
       else if( !strncmp(OPERATOR_PREFIX_TOFILEFREE, token->v,
                    OPERATOR_PREFIX_LENGTH_TOFILE) )
         arithmetic_tofile(p, token->v, 1);
-      else if( !strncmp(token->v, OPERATOR_PREFIX_SET,
-                        OPERATOR_PREFIX_LENGTH_SET) )
-        operands_set_name(p, token->v);
-      else if( gal_array_name_recognized(token->v)
-          || operands_is_name(p, token->v) )
+      else if( !strncmp(token->v, GAL_ARITHMETIC_SET_PREFIX,
+                        GAL_ARITHMETIC_SET_PREFIX_LENGTH) )
+        gal_arithmetic_set_name(&p->setprm, token->v);
+      else if(    gal_array_name_recognized(token->v)
+               || gal_arithmetic_set_is_name(p->setprm.named, token->v) )
         operands_add(p, token->v, NULL);
       else if( (data=gal_data_copy_string_to_number(token->v)) )
         {
@@ -1402,7 +1430,7 @@ reversepolish(struct arithmeticparams *p)
         }
 
       /* Increment the token counter. */
-      ++p->tokencounter;
+      ++p->setprm.tokencounter;
     }
 
 
@@ -1479,7 +1507,7 @@ reversepolish(struct arithmeticparams *p)
      into 'data', so it is freed when freeing 'data'. */
   gal_data_free(data);
   free(p->refdata.dsize);
-  gal_list_data_free(p->named);
+  gal_list_data_free(p->setprm.named);
 
 
   /* Clean up. Note that the tokens were taken from the command-line
