@@ -5,7 +5,7 @@ Arithmetic is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2020, Free Software Foundation, Inc.
+Copyright (C) 2015-2021, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -377,7 +377,8 @@ ui_preparations(struct arithmeticparams *p)
       dsize=gal_fits_img_info_dim(p->wcsfile, p->wcshdu, &ndim);
 
       /* Read the WCS. */
-      p->refdata.wcs=gal_wcs_read(p->wcsfile, p->wcshdu, 0, 0,
+      p->refdata.wcs=gal_wcs_read(p->wcsfile, p->wcshdu,
+                                  p->cp.wcslinearmatrix, 0, 0,
                                   &p->refdata.nwcs);
       if(p->refdata.wcs)
         {
