@@ -5,7 +5,7 @@ Fits is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2017-2020, Free Software Foundation, Inc.
+Copyright (C) 2017-2021, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -88,18 +88,18 @@ fits_print_extension_info(struct fitsparams *p)
 {
   uint16_t *ui16;
   fitsfile *fptr;
-  int hasblankname=0;
-  gal_data_t *cols=NULL, *tmp;
-  char **tstra, **estra, **sstra;
   size_t i, numext, *dsize, ndim;
+  int hascomments=0, hasblankname=0;
+  char **tstra, **estra, **sstra, **cmstra;
   int j, nc, numhdu, hdutype, status=0, type;
+  gal_data_t *tmp, *cols=NULL, *sizecol, *commentscol;
   char *msg, *tstr=NULL, sstr[1000], extname[FLEN_VALUE];
 
 
   /* Open the FITS file and read the first extension type, upon moving to
      the next extension, we will read its type, so for the first we will
      need to do it explicitly. */
-  fptr=gal_fits_hdu_open(p->filename, "0", READONLY);
+  fptr=gal_fits_hdu_open(p->input->v, "0", READONLY);
   if (fits_get_hdu_type(fptr, &hdutype, &status) )
     gal_fits_io_error(status, "reading first extension");
 
@@ -112,6 +112,8 @@ fits_print_extension_info(struct fitsparams *p)
 
   /* Allocate all the columns (in reverse order, since this is a simple
      linked list). */
+  gal_list_data_add_alloc(&cols, NULL, GAL_TYPE_STRING, 1, &numext, NULL, 1,
+                          -1, 1, "HDU_COMMENT", "note", "Possible comment");
   gal_list_data_add_alloc(&cols, NULL, GAL_TYPE_STRING, 1, &numext, NULL, 1,
                           -1, 1, "HDU_SIZE", "name", "Size of image or table "
                           "number of rows and columns.");
@@ -130,7 +132,12 @@ fits_print_extension_info(struct fitsparams *p)
   ui16  = cols->array;
   estra = cols->next->array;
   tstra = cols->next->next->array;
-  sstra = cols->next->next->next->array;
+
+  sizecol = cols->next->next->next;
+  sstra = sizecol->array;
+
+  commentscol = cols->next->next->next->next;
+  cmstra = commentscol->array;
 
   cols->next->disp_width=15;
   cols->next->next->disp_width=15;
@@ -182,6 +189,16 @@ fits_print_extension_info(struct fitsparams *p)
       status=0;
 
 
+      /* Check if its a healpix grid. */
+      if( gal_fits_hdu_is_healpix(fptr) )
+        {
+          hascomments=1;
+          gal_checkset_allocate_copy("HEALpix", cmstra+i);
+        }
+      else
+        gal_checkset_allocate_copy(GAL_BLANK_STRING, cmstra+i);
+
+
       /* Write the size into a string. 'sprintf' returns the number of
          written characters (excluding the '\0'). So for each dimension's
          size that is written, we add to 'nc' (the number of
@@ -229,12 +246,18 @@ fits_print_extension_info(struct fitsparams *p)
   /* Close the file. */
   fits_close_file(fptr, &status);
 
+  /* If there weren't any comments, don't print the comment column. */
+  if(hascomments==0)
+    {
+      sizecol->next=NULL;
+      gal_data_free(commentscol);
+    }
 
   /* Print the results. */
   if(!p->cp.quiet)
     {
       printf("%s\nRun on %s-----\n", PROGRAM_STRING, ctime(&p->rawtime));
-      printf("HDU (extension) information: '%s'.\n", p->filename);
+      printf("HDU (extension) information: '%s'.\n", p->input->v);
       printf(" Column 1: Index (counting from 0, usable with '--hdu').\n");
       printf(" Column 2: Name ('EXTNAME' in FITS standard, usable with "
              "'--hdu').\n");
@@ -244,6 +267,9 @@ fits_print_extension_info(struct fitsparams *p)
       printf(" Column 3: Image data type or 'table' format (ASCII or "
              "binary).\n");
       printf(" Column 4: Size of data in HDU.\n");
+      if(hascomments)
+        printf(" Column 5: Comments about the HDU (e.g., if its HEALpix, or "
+               "etc).\n");
       printf("-----\n");
     }
   gal_table_write(cols, NULL, NULL, GAL_TABLE_FORMAT_TXT, NULL, NULL, 0);
@@ -261,7 +287,7 @@ fits_hdu_number(struct fitsparams *p)
   int numhdu, status=0;
 
   /* Read the first extension (necessary for reading the rest). */
-  fptr=gal_fits_hdu_open(p->filename, "0", READONLY);
+  fptr=gal_fits_hdu_open(p->input->v, "0", READONLY);
 
   /* Get the number of HDUs. */
   if( fits_get_num_hdus(fptr, &numhdu, &status) )
@@ -281,22 +307,7 @@ fits_hdu_number(struct fitsparams *p)
 static void
 fits_datasum(struct fitsparams *p)
 {
-  int status=0;
-  fitsfile *fptr;
-  unsigned long datasum, hdusum;
-
-  /* Read the desired extension (necessary for reading the rest). */
-  fptr=gal_fits_hdu_open(p->filename, p->cp.hdu, READONLY);
-
-  /* Calculate the checksum and datasum of the opened HDU. */
-  fits_get_chksum(fptr, &datasum, &hdusum, &status);
-  gal_fits_io_error(status, "estimating datasum");
-
-  /* Close the file. */
-  fits_close_file(fptr, &status);
-
-  /* Print the datasum */
-  printf("%ld\n", datasum);
+  printf("%ld\n", gal_fits_hdu_datasum(p->input->v, p->cp.hdu));
 }
 
 
@@ -312,14 +323,15 @@ fits_pixelscale(struct fitsparams *p)
   double multip, *pixelscale;
 
   /* Read the desired WCS. */
-  wcs=gal_wcs_read(p->filename, p->cp.hdu, 0, 0, &nwcs);
+  wcs=gal_wcs_read(p->input->v, p->cp.hdu, p->cp.wcslinearmatrix,
+                   0, 0, &nwcs);
 
   /* If a WCS doesn't exist, let the user know and return. */
   if(wcs)
     ndim=wcs->naxis;
   else
     error(EXIT_FAILURE, 0, "%s (hdu %s): no WCS could be read by WCSLIB, "
-          "hence the pixel-scale cannot be determined", p->filename,
+          "hence the pixel-scale cannot be determined", p->input->v,
           p->cp.hdu);
 
   /* Calculate the pixel-scale in each dimension. */
@@ -332,7 +344,7 @@ fits_pixelscale(struct fitsparams *p)
     {
       printf("Basic information for --pixelscale (remove extra info "
              "with '--quiet' or '-q')\n");
-      printf("  Input: %s (hdu %s) has %zu dimensions.\n", p->filename,
+      printf("  Input: %s (hdu %s) has %zu dimensions.\n", p->input->v,
              p->cp.hdu, ndim);
       printf("  Pixel scale in each FITS dimension:\n");
       for(i=0;i<ndim;++i)
@@ -422,11 +434,11 @@ fits_skycoverage(struct fitsparams *p)
   double *center, *width, *min, *max;
 
   /* Find the coverage. */
-  if( gal_wcs_coverage(p->filename, p->cp.hdu, &ndim,
+  if( gal_wcs_coverage(p->input->v, p->cp.hdu, &ndim,
                        &center, &width, &min, &max)==0 )
     error(EXIT_FAILURE, 0, "%s (hdu %s): is not usable for finding "
           "sky coverage (either doesn't have a WCS, or isn't an image "
-          "or cube HDU with 2 or 3 dimensions", p->filename, p->cp.hdu);
+          "or cube HDU with 2 or 3 dimensions", p->input->v, p->cp.hdu);
 
   /* Inform the user. */
   if(p->cp.quiet)
@@ -442,7 +454,7 @@ fits_skycoverage(struct fitsparams *p)
     }
   else
     {
-      printf("Input file: %s (hdu: %s)\n", p->filename, p->cp.hdu);
+      printf("Input file: %s (hdu: %s)\n", p->input->v, p->cp.hdu);
       printf("\nSky coverage by center and (full) width:\n");
       switch(ndim)
         {
@@ -463,7 +475,8 @@ fits_skycoverage(struct fitsparams *p)
         }
 
       /* For the range type of coverage. */
-      wcs=gal_wcs_read(p->filename, p->cp.hdu, 0, 0, &nwcs);
+      wcs=gal_wcs_read(p->input->v, p->cp.hdu, p->cp.wcslinearmatrix,
+                       0, 0, &nwcs);
       printf("\nSky coverage by range along dimensions:\n");
       for(i=0;i<ndim;++i)
         printf("  %-8s %-15.10g%-15.10g\n", gal_wcs_dimension_name(wcs, i),
@@ -495,7 +508,7 @@ fits_hdu_remove(struct fitsparams *p, int *r)
       hdu=gal_list_str_pop(&p->remove);
 
       /* Open the FITS file at the specified HDU. */
-      fptr=gal_fits_hdu_open(p->filename, hdu, READWRITE);
+      fptr=gal_fits_hdu_open(p->input->v, hdu, READWRITE);
 
       /* Delete the extension. */
       if( fits_delete_hdu(fptr, &hdutype, &status) )
@@ -557,12 +570,12 @@ fits_hdu_copy(struct fitsparams *p, int cut1_copy0, int *r)
       hdu=gal_list_str_pop(&list);
 
       /* Open the FITS file at the specified HDU. */
-      in=gal_fits_hdu_open(p->filename, hdu,
+      in=gal_fits_hdu_open(p->input->v, hdu,
                            cut1_copy0 ? READWRITE : READONLY);
 
       /* If the output isn't opened yet, open it.  */
       if(out==NULL)
-        out = ( ( gal_fits_hdu_format(p->filename, hdu)==IMAGE_HDU
+        out = ( ( gal_fits_hdu_format(p->input->v, hdu)==IMAGE_HDU
                   && p->primaryimghdu )
                 ? fits_open_to_write_no_blank(p->cp.output)
                 : gal_fits_open_to_write(p->cp.output) );
