@@ -5,7 +5,7 @@ MakeCatalog is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2020, Free Software Foundation, Inc.
+Copyright (C) 2016-2021, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -538,7 +538,8 @@ ui_wcs_info(struct mkcatalogparams *p)
   size_t i;
 
   /* Read the WCS meta-data. */
-  p->objects->wcs=gal_wcs_read(p->objectsfile, p->cp.hdu, 0, 0,
+  p->objects->wcs=gal_wcs_read(p->objectsfile, p->cp.hdu,
+                               p->cp.wcslinearmatrix, 0, 0,
                                &p->objects->nwcs);
 
   /* Read the basic WCS information. */
@@ -1197,7 +1198,7 @@ ui_subtract_sky(struct mkcatalogparams *p)
           tile=&tl->tiles[tid];
 
           /* Subtract the Sky value from the input image. */
-          GAL_TILE_PARSE_OPERATE(tile, NULL, 0, 0, {*i-=skyarr[tid];});
+          GAL_TILE_PARSE_OPERATE(tile, p->values, 1, 0, {*o-=skyarr[tid];});
         }
     }
 
@@ -1207,10 +1208,6 @@ ui_subtract_sky(struct mkcatalogparams *p)
     error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to fix "
           "the problem. For some reason, the size doesn't match", __func__,
           PACKAGE_BUGREPORT);
-
-  /* Inform the user that this operation is done (if necessary). */
-  if(!p->cp.quiet)
-    printf("  - Sky subtracted from input values.\n");
 }
 
 
@@ -1427,10 +1424,20 @@ ui_preparations_read_keywords(struct mkcatalogparams *p)
           keys[0].array=&minstd;              keys[1].array=&p->medstd;
           gal_fits_key_read(p->usedstdfile, p->stdhdu, keys, 0, 0);
 
-          /* If the two keywords couldn't be read. We don't want to slow down
-             the user for the median (which needs sorting). So we'll just
-             calculate the minimum which is necessary for the 'p->cpscorr'. */
-          if(keys[1].status) p->medstd=NAN;
+          /* If the two keywords couldn't be read. We don't want to slow
+             down the user for the median (which needs sorting). So we'll
+             just calculate if if '--forcereadstd' is called. However, we
+             need the minimum for 'p->cpscorr'. */
+          if(keys[1].status)
+            {
+              if(p->forcereadstd)
+                {
+                  tmp=gal_statistics_median(p->std, 0);
+                  p->medstd=*((float *)(tmp->array));
+                }
+              else
+                p->medstd=NAN;
+            }
           if(keys[0].status)
             {
               /* Calculate the minimum STD. */
@@ -1909,16 +1916,20 @@ ui_read_check_inputs_setup(int argc, char *argv[], struct mkcatalogparams *p)
       if(p->subtractsky || p->sky)
         {
           if(p->sky->size==1)
-            printf("  - Sky: %g\n", *((float *)(p->sky->array)) );
+            printf("  - Sky: %g (single value for all pixels)\n",
+                   *((float *)(p->sky->array)) );
           else
             printf("  - Sky: %s (hdu: %s)\n", p->usedskyfile, p->skyhdu);
+          if(p->subtractsky)
+            printf("    - Sky has been subtracted from values internally.\n");
         }
 
       if(p->std)
         {
           tmp = p->variance ? "VAR" : "STD";
           if(p->std->size==1)
-            printf("  - Sky %s: %g\n", tmp, *((float *)(p->std->array)) );
+            printf("  - Sky %s: %g (single value for all pixels)\n", tmp,
+                   *((float *)(p->std->array)) );
           else
             printf("  - Sky %s: %s (hdu: %s)\n", tmp, p->usedstdfile,
                    p->stdhdu);
