@@ -5,7 +5,7 @@ NoiseChisel is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2020, Free Software Foundation, Inc.
+Copyright (C) 2015-2021, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -567,7 +567,8 @@ ui_preparations_read_input(struct noisechiselparams *p)
                                            NULL, GAL_TYPE_FLOAT32,
                                            p->cp.minmapsize,
                                            p->cp.quietmmap);
-  p->input->wcs = gal_wcs_read(p->inputname, p->cp.hdu, 0, 0,
+  p->input->wcs = gal_wcs_read(p->inputname, p->cp.hdu,
+                               p->cp.wcslinearmatrix, 0, 0,
                                &p->input->nwcs);
   p->input->ndim=gal_dimension_remove_extra(p->input->ndim,
                                             p->input->dsize,
@@ -848,12 +849,15 @@ ui_free_report(struct noisechiselparams *p, struct timeval *t1)
   free(p->maxtsize);
   free(p->maxltsize);
   free(p->cp.output);
-  if(p->skyname)          free(p->skyname);
-  if(p->detskyname)       free(p->detskyname);
-  if(p->qthreshname)      free(p->qthreshname);
-  if(p->detsn_s_name)     free(p->detsn_s_name);
-  if(p->detsn_d_name)     free(p->detsn_d_name);
-  if(p->detectionname)    free(p->detectionname);
+  if(p->khdu) free(p->khdu);
+  if(p->whdu) free(p->whdu);
+  if(p->chdu) free(p->chdu);
+  if(p->skyname) free(p->skyname);
+  if(p->detskyname) free(p->detskyname);
+  if(p->qthreshname) free(p->qthreshname);
+  if(p->detsn_s_name) free(p->detsn_s_name);
+  if(p->detsn_d_name) free(p->detsn_d_name);
+  if(p->detectionname) free(p->detectionname);
 
   /* Free the allocated datasets. */
   gal_data_free(p->sky);
@@ -863,6 +867,7 @@ ui_free_report(struct noisechiselparams *p, struct timeval *t1)
   gal_data_free(p->kernel);
   gal_data_free(p->binary);
   gal_data_free(p->olabel);
+  gal_data_free(p->noskytiles);
   gal_data_free(p->widekernel);
   if(p->conv!=p->input) gal_data_free(p->conv);
 
