@@ -47,7 +47,7 @@ struct argp_option program_options[] =
     {
       "wcsfile",
       UI_KEY_WCSFILE,
-      "STR",
+      "FITS",
       0,
       "File with WCS if conversion is requested.",
       GAL_OPTIONS_GROUP_INPUT,
@@ -73,7 +73,7 @@ struct argp_option program_options[] =
     {
       "catcolumnfile",
       UI_KEY_CATCOLUMNFILE,
-      "STR",
+      "FITS/TXT",
       0,
       "File(s) to be concatenated by column.",
       GAL_OPTIONS_GROUP_INPUT,
@@ -315,6 +315,46 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
+    {
+      "rowlimit",
+      UI_KEY_ROWLIMIT,
+      "INT,INT",
+      0,
+      "Only rows in this row-counter range.",
+      UI_GROUP_OUTROWS,
+      &p->rowlimit,
+      GAL_TYPE_STRING,
+      GAL_OPTIONS_RANGE_GE_0,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      gal_options_parse_csv_float64
+    },
+    {
+      "rowrandom",
+      UI_KEY_ROWRANDOM,
+      "INT",
+      0,
+      "Number of rows to select randomly.",
+      UI_GROUP_OUTROWS,
+      &p->rowrandom,
+      GAL_TYPE_SIZE_T,
+      GAL_OPTIONS_RANGE_GE_0,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+    },
+    {
+      "envseed",
+      UI_KEY_ENVSEED,
+      0,
+      0,
+      "Use GSL_RNG_SEED env. for '--rowrandom'.",
+      UI_GROUP_OUTROWS,
+      &p->envseed,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
     {
       "noblank",
       UI_KEY_NOBLANK,
