@@ -77,7 +77,7 @@ enum gal_arithmetic_operators
 
   GAL_ARITHMETIC_OP_PLUS,         /*   +     */
   GAL_ARITHMETIC_OP_MINUS,        /*   -     */
-  GAL_ARITHMETIC_OP_MULTIPLY,     /*   *     */
+  GAL_ARITHMETIC_OP_MULTIPLY,     /*   x     */
   GAL_ARITHMETIC_OP_DIVIDE,       /*   /     */
   GAL_ARITHMETIC_OP_MODULO,       /*   %     */
 
@@ -106,10 +106,26 @@ enum gal_arithmetic_operators
   GAL_ARITHMETIC_OP_LOG,          /* log()   */
   GAL_ARITHMETIC_OP_LOG10,        /* log10() */
 
-  GAL_ARITHMETIC_OP_RA_TO_DEGREE, /* right ascension to decimal      */
-  GAL_ARITHMETIC_OP_DEC_TO_DEGREE,/* declination to decimal          */
-  GAL_ARITHMETIC_OP_DEGREE_TO_RA, /* right ascension to decimal      */
-  GAL_ARITHMETIC_OP_DEGREE_TO_DEC,/* declination to decimal          */
+  GAL_ARITHMETIC_OP_SIN,          /* sine (input in deg).    */
+  GAL_ARITHMETIC_OP_COS,          /* cosine (input in deg).  */
+  GAL_ARITHMETIC_OP_TAN,          /* tangent (input in deg). */
+  GAL_ARITHMETIC_OP_ASIN,         /* Inverse sine (output in deg). */
+  GAL_ARITHMETIC_OP_ACOS,         /* Inverse cosine (output in deg). */
+  GAL_ARITHMETIC_OP_ATAN,         /* Inverse tangent (output in deg). */
+  GAL_ARITHMETIC_OP_ATAN2,        /* Inv. atan (preserves quad, out in deg). */
+  GAL_ARITHMETIC_OP_SINH,         /* Hyperbolic sine. */
+  GAL_ARITHMETIC_OP_COSH,         /* Hyperbolic cosine. */
+  GAL_ARITHMETIC_OP_TANH,         /* Hyperbolic tangent. */
+  GAL_ARITHMETIC_OP_ASINH,        /* Inverse hyperbolic sine. */
+  GAL_ARITHMETIC_OP_ACOSH,        /* Inverse hyperbolic cosine. */
+  GAL_ARITHMETIC_OP_ATANH,        /* Inverse hyperbolic tangent. */
+
+  GAL_ARITHMETIC_OP_RA_TO_DEGREE, /* right ascension to decimal. */
+  GAL_ARITHMETIC_OP_DEC_TO_DEGREE,/* declination to decimal. */
+  GAL_ARITHMETIC_OP_DEGREE_TO_RA, /* right ascension to decimal. */
+  GAL_ARITHMETIC_OP_DEGREE_TO_DEC,/* declination to decimal. */
+  GAL_ARITHMETIC_OP_COUNTS_TO_MAG,/* Counts to magnitude. */
+  GAL_ARITHMETIC_OP_COUNTS_TO_JY, /* Counts to Janskys with AB-mag zeropoint. */
 
   GAL_ARITHMETIC_OP_MINVAL,       /* Minimum value of array.               */
   GAL_ARITHMETIC_OP_MAXVAL,       /* Maximum value of array.               */
